#!/usr/bin/ruby -wW2d

require 'etc'
require 'twitter'
require 'net/http'
require 'libnotify'

require_relative 'config'

module Twitter
  class Notifier

    #
    # Initialize
    #
    # @return void
    #
    def initialize

      init_data_storage

      log "****************************"
      log "Starting Twitter Notifier..."

      @statuses = []
      @last_status_id = nil

      Libnotify.show(
          :body => "Starting Twitter Notifier",
          :summary => "Twitter notifier",
          :timeout => 2
      )

      init_listeners

    end

    #
    # Begin the listener threads
    #
    # @return void
    #
    def init_listeners

      Thread.new do
        while true
          get_newest_tweets
          sleep 15
        end
      end

      Thread.new do
        while true

          while @statuses.length > 0
            tweet      = @statuses.pop
            image_path = tweet.user.get_image @icon_dir

            Libnotify.show(
                :body      => "#{tweet.text}\n#{tweet.created_at.to_time}",
                :summary   => "@#{tweet.user.screen_name}",
                :icon_path => image_path,
                :timeout   => 5
            )

            self.log "Twitter: @#{tweet.user.screen_name}: #{tweet.text}"
            sleep 2
          end
          sleep 2

        end
      end

    end

    #
    # Retrieve the latest tweets
    #
    # @return void
    #
    def get_newest_tweets

      begin
        if @last_status_id
          response = Twitter.home_timeline(:since_id => @last_status_id)
        else
          response = Twitter.home_timeline(:count => 5)
        end
      rescue => exc
        self.log "#{exc.class}: #{exc.message}"
        return
      end

      if response.first
        self.log "Retrieved #{response.length} new tweet(s)."

        @last_status_id = response.first.id

        response.reverse.each do |tweet|
          @statuses.unshift(tweet)
        end

      else
        #self.log "No new tweets."
      end
    end

    #
    # Logs a message to the application log and the cli output
    #
    # @param message string Message to be logged
    #
    # @return void
    #
    def log(message)
      log_message = "[#{Time.parse(Time.now.to_s)}]: " + message + "\n"

      puts log_message

      File.open(@log_file, 'a') {|f| f.write(log_message)}
    end

    #
    # Initialize data storage directories and related instance variables
    #
    # @return void
    #
    def init_data_storage
      @data_dir = Etc.getpwuid.dir + "/.twitter-notifier"
      @icon_dir = @data_dir + "/twitter-images"
      @log_dir  = @data_dir + "/log"
      @log_file = @log_dir + "/activity.log"

      Dir::mkdir @data_dir unless FileTest::directory? @data_dir
      Dir::mkdir @icon_dir unless FileTest::directory? @icon_dir or not FileTest::directory? @data_dir
      Dir::mkdir @log_dir  unless FileTest::directory? @log_dir or not FileTest::directory? @data_dir

    end

  end

  #
  # Modifications to [Twitter::User]
  #
  class User

    #
    # Retrieves the full path to local image, and the image file, if necessary
    #
    # @param base_path string Base path to the images folder
    #
    # @return string
    #
    def get_image(base_path)
      @base_path = base_path if base_path

      if FileTest::file? image_path and (FileTest::size image_path).to_i > 0
        # if image exists, download asynchronously
        Thread.new {store_image}
      else
        # synchronously
        store_image
      end

      image_path

    end

    #
    # returns the full image path
    #
    # @return string
    #
    def image_path
      uri_path  = URI(profile_image_url).path
      extension = File.extname(uri_path)
      "#{@base_path}/#{screen_name}#{extension}"
    end

    private

    #
    # Downloads and saves the user's profile image.
    #
    def store_image
      uri = URI.parse(profile_image_url)
      Net::HTTP.start(uri.host, uri.port) do |http|
        resp = http.get(uri.path)
        open(image_path, "wb") do |file|
          file.write(resp.body)
        end
      end
    end

  end
end

Twitter::Notifier.new

sleep